from io import BytesIO, StringIO
from pathlib import Path

from .handlers import file_handlers, register_handler  # noqa: F401
from mmconf.utils import mkdir_or_exist

backend_instances: dict = {}


def is_str(s):
    """Check if an object is a string."""
    return isinstance(s, str)


def load(file,
         file_format=None,
         encoding='utf-8',
         **kwargs):
    """Load data from json/yaml/pickle files.

    This method provides a unified api for loading data from serialized files.

    ``load`` supports loading data from serialized files those can be storaged
    in different backends.

    Args:
        file (str or :obj:`Path` or file-like object): Filename or a file-like
            object.
        file_format (str, optional): If not specified, the file format will be
            inferred from the file extension, otherwise use the specified one.
            Currently supported formats include "json", "yaml/yml" and
            "pickle/pkl".
        encoding (str, optional): Encoding of the file. Defaults to 'utf-8'.

    Returns:
        The content from the file.
    """
    if isinstance(file, Path):
        file = str(file)
    if file_format is None and is_str(file):
        file_format = file.split('.')[-1]
    if file_format not in file_handlers:
        raise TypeError(f'Unsupported format: {file_format}')

    handler = file_handlers[file_format]
    if is_str(file):
        with open(file, encoding=encoding) as f:
            text = f.read()

        with StringIO(text) as f:
            obj = handler.load_from_fileobj(f, **kwargs)

    elif hasattr(file, 'read'):
        obj = handler.load_from_fileobj(file, **kwargs)
    else:
        raise TypeError('"file" must be a filepath str or a file-object')
    return obj


def dump(obj,
         file=None,
         encoding='utf-8',
         file_format=None,
         **kwargs):
    """Dump data to json/yaml/pickle strings or files.

    This method provides a unified api for dumping data as strings or to files,
    and also supports custom arguments for each file format.

    ``dump`` supports dumping data as strings or to files which is saved to
    different backends.

    Args:
        obj (any): The python object to be dumped.
        file (str or :obj:`Path` or file-like object, optional): If not
            specified, then the object is dumped to a str, otherwise to a file
            specified by the filename or file-like object.
        file_format (str, optional): Same as :func:`load`.
        encoding (str, optional): Encoding of the file. Defaults to 'utf-8'.
        file_client_args (dict, optional): Arguments to instantiate a
            FileClient. See :class:`mmengine.fileio.FileClient` for details.
            Defaults to None. It will be deprecated in future. Please use
            ``backend_args`` instead.

    Examples:
        >>> dump('hello world', '/path/of/your/file')  # disk
        >>> dump('hello world', 's3://path/of/your/file')  # ceph or petrel

    Returns:
        bool: True for success, False otherwise.
    """
    if isinstance(file, Path):
        file = str(file)
    if file_format is None:
        if is_str(file):
            file_format = file.split('.')[-1]
        elif file is None:
            raise ValueError(
                'file_format must be specified since file is None')
    if file_format not in file_handlers:
        raise TypeError(f'Unsupported format: {file_format}')

    handler = file_handlers[file_format]
    if file is None:
        return handler.dump_to_str(obj, **kwargs)
    elif is_str(file):
        with StringIO() as f:
            handler.dump_to_fileobj(obj, f, **kwargs)
            mkdir_or_exist(Path(file).parent)
            with open(file, 'w', encoding=encoding) as f:
                f.write(obj)

    elif hasattr(file, 'write'):
        handler.dump_to_fileobj(obj, file, **kwargs)
    else:
        raise TypeError('"file" must be a filename str or a file-object')


from .version_utils import digit_version, get_git_hash
from .path import (check_file_exist, fopen, is_abs, is_filepath,
                   mkdir_or_exist, scandir, symlink)
from .package_utils import (call_command, get_installed_path, install_package,
                            is_installed)
from .misc import (deprecated_api_warning, deprecated_function,
                   get_object_from_string, import_modules_from_strings,
                   is_list_of, is_seq_of, is_str, is_tuple_of,
                   iter_cast, list_cast, to_1tuple, to_2tuple, to_3tuple, to_4tuple,
                   to_ntuple, tuple_cast)
